//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  ClarinetTab Plugin
//
//  Copyright (C)2010 Nicolas Froment (lasconic)
//  Copyright (C)2015 Sylvain Kuntzmann (skunt)
//  Copyright (C)2015 Hervé Laurent (AirW)
//  Copyright (C)2016 Matthieu Giroux (Matthius)
//  https://bitbucket.org/matthieugiroux/musescore-clarinet/downloads
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//=============================================================================

import QtQuick 2.0
import MuseScore 1.0

MuseScore {
    version: "1.0"
    description: qsTr("This plugin displays fingering for clarinet in C")
    menuPath: "Plugins.Notes." + qsTr("ClarinetTab")

    property variant fingerings : [ "q", "w", "e", "r" , "t", "y", "u", "i",
    "o", "p", "a", "s", "d", "f", "g", "h", "j", "k", "l",
									"Q", "W", "E", "R" , "T", "Y", "U", "I",
    "O", "P", "A", "S", "D", "F", "Z", "G", "H", "J", "K" ];

    onRun: {
        apply();
        Qt.quit();
    }

    function tabNotes(notes, cursor) {
        var pitch = notes[0].pitch;  // pitch of the chord top note
        var index = pitch - 52;      //  index =  pitch - 52 (59 is the midi pitch of Cm, 0 index in fingerings)
        if(index >= 0 && index < fingerings.length){
            var text = newElement(Element.STAFF_TEXT);
            text.text = '<font face="ClarinetTab"/>' + '<font size="40"/>' + fingerings[index];
            text.userOff.y = -7,5;
            text.userOff.x = -1,1;
            cursor.add(text);
        }
    }

    function applyToSelection(func) {
        if (typeof curScore === 'undefined')
            Qt.quit();
        var cursor = curScore.newCursor();
        var startStaff;
        var endStaff;
        var endTick;
        var fullScore = false;
        cursor.rewind(1);
        if (!cursor.segment) { // no selection
            fullScore = true;
            startStaff = 0; // start with 1st staff
            endStaff  = curScore.nstaves - 1; // and end with last
        } else {
            startStaff = cursor.staffIdx;
            cursor.rewind(2);
            if (cursor.tick == 0) {
                // this happens when the selection includes
                // the last measure of the score.
                // rewind(2) goes behind the last segment (where
                // there's none) and sets tick=0
                endTick = curScore.lastSegment.tick + 1;
            } else {
                endTick = cursor.tick;
            }
            endStaff   = cursor.staffIdx;
        }
        console.log(startStaff + " - " + endStaff + " - " + endTick)

        for (var staff = startStaff; staff <= endStaff; staff++) {
            for (var voice = 0; voice < 4; voice++) {
                cursor.rewind(1); // beginning of selection
                cursor.voice    = voice;
                
                cursor.staffIdx = staff;

                if (fullScore)  // no selection
                    cursor.rewind(0); // beginning of score
                    while (cursor.segment && (fullScore || cursor.tick < endTick)) {
                        if (cursor.visible != 0 && cursor.element && cursor.element.type == Element.CHORD) {
                            var graceChords = cursor.element.graceNotes;
                            for (var i = 0; i < graceChords.length; i++) {
                                // iterate through all grace chords
                                var notes = graceChords[i].notes;
                                func(notes, cursor);
                            }
                            var notes = cursor.element.notes;
                            func(notes, cursor);
                        } // end if CHORD
                        cursor.next();
                    } // end while segment
            } // end for voice
        } // end for staff
        Qt.quit();
    } // end applyToSelection()

    function apply() {
        applyToSelection(tabNotes)
    }

}

